@@ -4,22 +4,21 @@ import pathlib
 import re
 import shutil
 import sys
-from unittest.mock import patch, Mock, call
+from unittest.mock import Mock, call, patch
 
+import autoapi.settings
 import pytest
 import sphinx
-from sphinx.application import Sphinx
-from sphinx.errors import ExtensionError
 import sphinx.util.logging
-
 from autoapi.mappers.python import (
-    PythonModule,
-    PythonFunction,
     PythonClass,
     PythonData,
+    PythonFunction,
     PythonMethod,
+    PythonModule,
 )
-import autoapi.settings
+from sphinx.application import Sphinx
+from sphinx.errors import ExtensionError
 
 
 def rebuild(confoverrides=None, confdir=".", **kwargs):
@@ -30,7 +29,7 @@ def rebuild(confoverrides=None, confdir=".", **kwargs):
         doctreedir="_build/.doctrees",
         buildername="text",
         confoverrides=confoverrides,
-        **kwargs
+        **kwargs,
     )
     app.build()
 
@@ -104,6 +103,15 @@ class TestSimpleModule:
         # (autoapi_python_class_content="both") class docstring only once.
         assert example_file.count("One __init__.") == 3
 
+        # Tuples should be rendered as tuples, not lists
+        assert "('a', 'b')" in example_file
+        # Lists should be rendered as lists, not tuples
+        assert "['c', 'd']" in example_file
+
+        # Assigning a class level attribute at the module level
+        # should not get documented as a module level attribute.
+        assert "dinglebop" not in example_file
+
         index_path = "_build/text/index.txt"
         with io.open(index_path, encoding="utf8") as index_handle:
             index_file = index_handle.read()
@@ -251,10 +259,13 @@ class TestPy3Module:
         with io.open(example_path, encoding="utf8") as example_handle:
             example_file = example_handle.read()
 
-        assert "software = sphinx" in example_file
+        assert '''software = "sphin'x"''' in example_file
+        assert """more_software = 'sphinx"autoapi'""" in example_file
+        assert "interesting_string = 'interesting\"fun\\'\\\\\\'string'" in example_file
+
         assert "code_snippet = Multiline-String" in example_file
 
-        assert "max_rating :int = 10" in example_file
+        assert "max_rating: int = 10" in example_file
         assert "is_valid" in example_file
 
         assert "ratings" in example_file
@@ -265,7 +276,10 @@ class TestPy3Module:
         assert "start: int" in example_file
         assert "Iterable[int]" in example_file
 
-        assert "List[Union[str, int]]" in example_file
+        if sys.version_info >= (3, 8):
+            assert "List[str | int]" in example_file
+        else:
+            assert "List[Union[str, int]]" in example_file
 
         assert "not_yet_a: A" in example_file
         assert "imported: example2.B" in example_file
@@ -276,7 +290,7 @@ class TestPy3Module:
 
         assert "instance_var" in example_file
 
-        assert "global_a :A" in example_file
+        assert "global_a: A" in example_file
 
         assert "my_method() -> str" in example_file
 
@@ -345,7 +359,7 @@ class TestAnnotationCommentsModule:
         with io.open(example_path, encoding="utf8") as example_handle:
             example_file = example_handle.read()
 
-        assert "max_rating :int = 10" in example_file
+        assert "max_rating: int = 10" in example_file
 
         assert "ratings" in example_file
         assert "List[int]" in example_file
@@ -357,7 +371,10 @@ class TestAnnotationCommentsModule:
         # assert "end: int" in example_file
         assert "Iterable[int]" in example_file
 
-        assert "List[Union[str, int]]" in example_file
+        if sys.version_info >= (3, 8):
+            assert "List[str | int]" in example_file
+        else:
+            assert "List[Union[str, int]]" in example_file
 
         assert "not_yet_a: A" in example_file
         assert "is_an_a" in example_file
@@ -365,7 +382,7 @@ class TestAnnotationCommentsModule:
 
         assert "instance_var" in example_file
 
-        assert "global_a :A" in example_file
+        assert "global_a: A" in example_file
 
         assert "class example.B(a: str)" in example_file
         assert "method(b: list)" in example_file
@@ -388,18 +405,32 @@ class TestPositionalOnlyArgumentsModule:
 
         assert "f_simple(a, b, /, c, d, *, e, f)" in example_file
 
-        assert (
-            "f_comment(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
-            in example_file
-        )
-        assert (
-            "f_annotation(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
-            in example_file
-        )
-        assert (
-            "f_arg_comment(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
-            in example_file
-        )
+        if sys.version_info >= (3, 8):
+            assert (
+                "f_comment(a: int, b: int, /, c: int | None, d: int | None, *, e: float, f: float)"
+                in example_file
+            )
+            assert (
+                "f_annotation(a: int, b: int, /, c: int | None, d: int | None, *, e: float, f: float)"
+                in example_file
+            )
+            assert (
+                "f_arg_comment(a: int, b: int, /, c: int | None, d: int | None, *, e: float, f: float)"
+                in example_file
+            )
+        else:
+            assert (
+                "f_comment(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
+                in example_file
+            )
+            assert (
+                "f_annotation(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
+                in example_file
+            )
+            assert (
+                "f_arg_comment(a: int, b: int, /, c: Optional[int], d: Optional[int], *, e: float, f: float)"
+                in example_file
+            )
         assert "f_no_cd(a: int, b: int, /, *, e: float, f: float)" in example_file
 
 
@@ -903,23 +934,23 @@ def test_string_module_attributes(builder):
 
     code_snippet_contents = [
         ".. py:data:: code_snippet",
-        "   :annotation: = Multiline-String",
+        "   :value: Multiline-String",
         "",
         "    .. raw:: html",
         "",
         "        <details><summary>Show Value</summary>",
         "",
-        "    .. code-block:: text",
-        "        :linenos:",
+        "    .. code-block:: python",
         "",
+        '        """The following is some code:',
         "        ",  # <--- Line array monstrosity to preserve these leading spaces
         "        # -*- coding: utf-8 -*-",
         "        from __future__ import absolute_import, division, print_function, unicode_literals",
         "        # from future.builtins.disabled import *",
         "        # from builtins import *",
-        "",
+        "        ",
         """        print("chunky o'block")""",
-        "",
+        '        """',
         "",
         "    .. raw:: html",
         "",
@@ -966,3 +997,29 @@ class TestMdSource:
             warningiserror=True,
             confoverrides={"source_suffix": ["md"]},
         )
+
+
+class TestMemberOrder:
+    @pytest.fixture(autouse=True, scope="class")
+    def built(self, builder):
+        builder(
+            "pyexample",
+            warningiserror=True,
+            confoverrides={
+                "suppress_warnings": ["app"],
+                "autodoc_member_order": "bysource",
+            },
+        )
+
+    def test_line_number_order(self):
+        example_path = "_build/text/manualapi.txt"
+        with io.open(example_path, encoding="utf8") as example_handle:
+            lines = example_handle.readlines()
+
+        method_tricky_pos = lines.index(
+            "   method_tricky(foo=None, bar=dict(foo=1, bar=2))\n"
+        )
+        method_sphinx_docs_pos = lines.index("   method_sphinx_docs(foo, bar=0)\n")
+
+        # method_tricky is defined in the source before method_sphinx_docs
+        assert method_tricky_pos < method_sphinx_docs_pos
